package com.bs.logisticslnfoapi.aspect;

import com.bs.logisticslnfoapi.bean.ResultBean;
import com.bs.logisticslnfoapi.exceptions.APIBusinessException;
import com.bs.logisticslnfoapi.exceptions.BusinessException;
import com.bs.logisticslnfoapi.util.Md5Util;
import lombok.extern.slf4j.Slf4j;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

/**
 * @ClassName ControllerAspect
 * @Description TODO
 * @Author Administrator
 * @Date 2019/8/5 0005 上午 11:31
 * @Version 1.0
 **/
@Component
@Aspect
@Slf4j
public class ControllerAspect {

    @Value("${hdappKey}")
    private String appkey;

    @Value("${hdappScrect}")
    private String appscrect;

    @Value("${hdversion}")
    private String version;

    @Around("execution(* com.bs.logisticslnfoapi.controller.JhyController.addPhone(..))")
    public Object around(ProceedingJoinPoint pjp){
        long startTime = System.currentTimeMillis();
        ResultBean<?> result;
        try {
            Object[] args = pjp.getArgs();
            //参数验证
            paramVerify(args);
            result = (ResultBean<?>) pjp.proceed();
            log.info(pjp.getSignature() + "use time:" + (System.currentTimeMillis() - startTime));
        } catch (Throwable e) {
            result = handlerException(pjp, e);
        }
        return result;
    }

    /**
     * 参数验证
     * @param args
     * @throws Exception
     */
    private void paramVerify(Object[] args) throws Exception {
        if(args[0] == null){
            throw new APIBusinessException("appkey不能为空");
        }else if(args[1] == null){
            throw new APIBusinessException("version不能为空");
        }else if(args[2] == null){
            throw new APIBusinessException("data不能为空");
        }else if(args[3] == null){
            throw new APIBusinessException("sign不能为空");
        }else if(!appkey.equals(args[0].toString())){
            throw new APIBusinessException("appkey错误");
        }else if(!version.equals(args[1].toString())){
            throw new APIBusinessException("version错误");
        }else if(!Md5Util.doSign(args[2].toString()+appscrect).equals(args[3])){
            throw new APIBusinessException("sign错误");
        }
        log.info("paramVerify----------------->：参数验证成功");
    }

    private ResultBean<?> handlerException(ProceedingJoinPoint pjp, Throwable e) {
        ResultBean<?> result = new ResultBean();

        // 已知异常
        if (e instanceof APIBusinessException){
            log.error(pjp.getSignature() + " error ", e);
            result.setMsg(e.getMessage());
            result.setCode(ResultBean.FAIL);
        } else if(e instanceof BusinessException){
            log.error(pjp.getSignature() + " error ", e);
            result.setMsg(e.toString());
            result.setCode(ResultBean.FAIL);
        } else if(e instanceof Exception){
            log.error(pjp.getSignature() + " error ", e);
            result.setMsg("系统错误");
            result.setCode(ResultBean.SYSFAIL);
        }
        return result;
    }
}
