package com.bs.logisticslnfoapi.bean.cainiao;

import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;

import java.util.List;

/**
 * @PackageName: cn.haliaeetus.sendmessage.bean.cainiao
 * @ClassName: Trace
 * @Description: TODO
 * @Author: liujs
 * @Date: 2020/02/24 下午 14:18
 * @Version: 1.0
 */
@Data
@Builder
@NoArgsConstructor
@AllArgsConstructor
public class Trace {
    /**
     * 国家（默认中国）  英文回传（默认请回传China）
     */
    private String country;
    /**
     * 省份
     */
    private String province;
    /**
     * 二级地址。举例：杭州市
     */
    private String city;
    /**
     * 站点类型(3:代收点，9:自提柜)
     */
    private String facilityType;
    /**
     * 网点名称
     * 举例：金宝商务大厦代收点
     */
    private String facilityName;
    /**
     * 代收点编号
     */
    private String facilityNo;
    /**
     * 操作时间
     * 格式yyyy-mm-dd hh:mm:ss。
     * 举例：2019-11-11 11:11:11
     */
    private String time;
    /**
     * 跟踪信息描述
     * 物流详情展示的原始信息。
     * 举例：[代收点]您的快件已由金宝商务大厦代收点代收，代收点地址：广东省深圳市宝安区顺丰路18号，联系电话：18566059116
     */
    private String desc;
    /**
     * 事件/操作
     * 三方代收点能合法回传的action如下：
     * SH_INBOUND 代收点已代收
     * SH_FAILED 代收点拒收（指的是消费者拒收）
     * SH_SIGNED 代收点签收（指的是消费取出包裹）
     * SH_MAILMAN_PICKUP 小件员从代收点将包裹取出
     * SH_MOVE_PACK 代收点包裹移库单行代码
     */
    private String action;
    /**
     * 代收点电话 (条件必选)
     * SH_INBOUND 必选。举例：18566059116； SH_MAILMAN_PICKUP 必选。该节点下回传小件员手机号
     */
    private String contactPhone;
    /**
     * 代收点地址 (条件必选)
     * SH_INBOUND、SH_MOVE_PACK 必选。
     * 举例：广东省深圳市宝安区顺丰路18号
     */
    private String address;
    /**
     * 时区,默认值：+8
     */
    private String tz;
    /**
     * 备注信息（SH_FAILED节点使用）
     * 格式样例：
     * <remark>SEC_CODE^^^140,150~~~REASON_DESC^^^联系不上用户</remark>
     */
    private String remark;
    /**
     * 拓展字段
     */
    private List<ExtendField> extendFields;
    /**
     * 小件取件员  action("SH_MAILMAN_PICKUP")才需要该字段
     */
    private String contacter;
}
