package com.bs.logisticslnfoapi.controller;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.aliyuncs.CommonRequest;
import com.aliyuncs.CommonResponse;
import com.aliyuncs.DefaultAcsClient;
import com.aliyuncs.IAcsClient;
import com.aliyuncs.exceptions.ClientException;
import com.aliyuncs.http.MethodType;
import com.aliyuncs.http.ProtocolType;
import com.aliyuncs.profile.DefaultProfile;
import com.aliyuncs.profile.IClientProfile;
import com.aliyuncs.sts.model.v20150401.AssumeRoleRequest;
import com.aliyuncs.sts.model.v20150401.AssumeRoleResponse;
import com.bs.logisticslnfoapi.bean.ApiLog;
import com.bs.logisticslnfoapi.bean.alibb.AlibbQuerySmsDetailsRequest;
import com.bs.logisticslnfoapi.bean.alibb.AlibbSendSmsRequest;
import com.bs.logisticslnfoapi.bean.alibb.OssTokenBean;
import com.bs.logisticslnfoapi.bean.alibb.OssTokenRequestBean;
import com.bs.logisticslnfoapi.service.log.LogService;
import com.bs.logisticslnfoapi.util.Enum.PlatformEnum;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Date;

/**
 * 阿里巴巴相关接口<br>
 *
 * @className: Alibb
 * @package: com.bs.logisticslnfoapi.controller
 * @author: kael
 * @date: 2019/10/21 0021 上午 11:10
 */
@RestController
@Slf4j
@CrossOrigin("*")
@RequestMapping(value ="/logistics/alibb")
public class AlibbController {

    // 目前只有"cn-hangzhou"这个region可用, 不要使用填写其他region的值
    public static final String REGION_CN_HANGZHOU = "cn-hangzhou";
    public static final String STS_API_VERSION = "2015-04-01";

    @Autowired
    private LogService logService;

    /**
     * 阿里巴巴短信发送接口
     * @param request
     * @return
     * @throws Exception
     */
    @RequestMapping("/sendSms")
    public Object alibbSendSms(@RequestBody AlibbSendSmsRequest request) throws Exception{
        long startTime = System.currentTimeMillis();
        log.info("【Alibb短信发送接口传入参数】：{}",JSON.toJSONString(request));
        DefaultProfile profile = DefaultProfile.getProfile("cn-hangzhou", request.getAccessKeyId(), request.getAppSecret());
        IAcsClient client = new DefaultAcsClient(profile);
        CommonRequest commonRequest = new CommonRequest();
        //系统参数
        commonRequest.setDomain(request.getUrl());
        commonRequest.setVersion("2017-05-25");
        commonRequest.setAction("SendSms");
        //业务参数
        commonRequest.putQueryParameter("RegionId", "cn-hangzhou");
        commonRequest.putQueryParameter("PhoneNumbers",request.getPhoneNumbers());
        commonRequest.putQueryParameter("SignName",request.getSignName());
        commonRequest.putQueryParameter("TemplateCode",request.getTemplateCode());
        commonRequest.putQueryParameter("TemplateParam",JSON.toJSONString(request.getTemplateParam()));
        CommonResponse response = client.getCommonResponse(commonRequest);
        log.info("【Alibb短信发送接口返回参数】：{}",response.getData());
        addLog(startTime, JSON.toJSONString(request), response,request.getUrl());
        return response.getData();
    }

    /**
     * 阿里巴巴短信结果查询
     * @param request
     * @return
     * @throws Exception
     */
    @RequestMapping("/querySendDetails")
    public Object alibbSmsQueryDetails(@RequestBody AlibbQuerySmsDetailsRequest request) throws Exception{
        long startTime = System.currentTimeMillis();
        log.info("【Alibb短信查询接口传入参数】：{}",JSON.toJSONString(request));

        DefaultProfile profile = DefaultProfile.getProfile("cn-hangzhou", request.getAccessKeyId(), request.getAppSecret());
        IAcsClient client = new DefaultAcsClient(profile);
        CommonRequest commonRequest = new CommonRequest();
        //系统参数
        commonRequest.setMethod(MethodType.POST);
        commonRequest.setDomain(request.getUrl());
        commonRequest.setVersion("2017-05-25");
        commonRequest.setAction("QuerySendDetails");
        commonRequest.putQueryParameter("RegionId", "cn-hangzhou");
        commonRequest.putQueryParameter("CurrentPage",request.getCurrentPage().toString());
        commonRequest.putQueryParameter("PageSize",request.getPageSize().toString());
        commonRequest.putQueryParameter("PhoneNumber",request.getPhoneNumber());
        commonRequest.putQueryParameter("SendDate",request.getSendDate());
        commonRequest.putQueryParameter("AccessKeyId",request.getAccessKeyId());
        commonRequest.putQueryParameter("Action","QuerySendDetails");
        commonRequest.putQueryParameter("BizId",request.getBizId());

        CommonResponse response = client.getCommonResponse(commonRequest);
        log.info("【Alibb短信回执接口返回参数】：{}",response.getData());
        addLog(startTime, JSON.toJSONString(request), response,request.getUrl());
        return response.getData();
    }

    /**
     * APP获取OSS临时token接口
     * @return
     * @throws Exception
     */
    @PostMapping("/ossSecurityToken")
    public Object ossSecurityToken(@RequestBody OssTokenRequestBean ossTokenRequestBean) throws Exception{
        try {
            ossTokenRequestBean.setProtocolType(ProtocolType.HTTPS);
            final AssumeRoleResponse stsResponse = assumeRole(ossTokenRequestBean.getAccessKeyId(), ossTokenRequestBean.getAccessKeySecret(), ossTokenRequestBean.getRoleArn(), ossTokenRequestBean.getRoleSessionName(),
                    ossTokenRequestBean.getPolicy(), ossTokenRequestBean.getProtocolType(), ossTokenRequestBean.getDurationSeconds());
            log.info("【Alibb APP获取OSS临时token接口返回参数】：{}",JSONObject.toJSONString(stsResponse.getCredentials()));
            return OssTokenBean.builder()
                    .statusCode("200")
                    .accessKeyId(stsResponse.getCredentials().getAccessKeyId())
                    .accessKeySecret(stsResponse.getCredentials().getAccessKeySecret())
                    .securityToken(stsResponse.getCredentials().getSecurityToken())
                    .expiration(stsResponse.getCredentials().getExpiration())
                    .build();
        } catch (ClientException e) {
            return OssTokenBean.builder()
                    .statusCode("500")
                    .ErrorCode(e.getErrCode())
                    .ErrorMessage(e.getErrMsg())
                    .build();
        }
    }

    protected AssumeRoleResponse assumeRole(String accessKeyId, String accessKeySecret, String roleArn,
                                            String roleSessionName, String policy, ProtocolType protocolType, long durationSeconds) throws ClientException
    {
        try {
            // 创建一个 Aliyun Acs Client, 用于发起 OpenAPI 请求
            IClientProfile profile = DefaultProfile.getProfile(REGION_CN_HANGZHOU, accessKeyId, accessKeySecret);
            DefaultAcsClient client = new DefaultAcsClient(profile);

            // 创建一个 AssumeRoleRequest 并设置请求参数
            final AssumeRoleRequest request = new AssumeRoleRequest();
            request.setVersion(STS_API_VERSION);
            request.setMethod(MethodType.POST);
            request.setProtocol(protocolType);

            request.setRoleArn(roleArn);
            request.setRoleSessionName(roleSessionName);
            request.setPolicy(policy);
            request.setDurationSeconds(durationSeconds);

            // 发起请求，并得到response
            final AssumeRoleResponse response = client.getAcsResponse(request);

            return response;
        } catch (ClientException e) {
            throw e;
        }
    }

    /**
     * 阿里调用接口新增日志
     * @param startTime
     * @param request
     * @param response
     */
    private void addLog(long startTime, String request, CommonResponse response,String url) {
        logService.install(ApiLog.builder()
                .requestMsg(request)
                .responseMsg(response == null ? "null" : response.getData())
                .url(url)
                .exTime(new Long((System.currentTimeMillis() - startTime)).intValue())
                .createTime(new Date())
                .requestType(1)
                .platform(PlatformEnum.ALBB.getEnValue())
                .build());
    }
}
