package com.bs.logisticslnfoapi.controller;

import com.alibaba.fastjson.JSON;
import com.bs.logisticslnfoapi.bean.ApiLog;
import com.bs.logisticslnfoapi.bean.cainiao.CaiNiaoReceiveRequest;
import com.bs.logisticslnfoapi.bean.cainiao.CaiNiaoRequest;
import com.bs.logisticslnfoapi.service.log.LogService;
import com.bs.logisticslnfoapi.util.Enum.PlatformEnum;
import com.bs.logisticslnfoapi.util.HttpUtils;
import com.bs.logisticslnfoapi.util.Md5Util;
import com.bs.logisticslnfoapi.util.OkHttpUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.http.NameValuePair;
import org.apache.http.message.BasicNameValuePair;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * 〈功能概述〉<br>
 *
 * @className: CaiNiaoController
 * @package: com.bs.logisticslnfoapi.controller
 * @author: kael
 * @date: 2019/11/28 0028 下午 17:03
 */
@RestController
@Slf4j
@CrossOrigin("*")
@RequestMapping(value = "/logistics/cainiao")
public class CaiNiaoController {

    @Autowired
    private LogService logService;

    @Value("${sendmessageurl}")
    private String sendmessageurl;

    /**
     * 菜鸟电子面单扩展信息查询
     *
     * @param request
     * @param response
     * @param caiNiaoRequest
     * @return
     * @throws Exception
     */
    @RequestMapping("/waybillExtendInfoQuery")
    public Object caiNiaoWaybillExtendInfoQuery(HttpServletRequest request, HttpServletResponse response, @RequestBody CaiNiaoRequest caiNiaoRequest) throws Exception {
        long startTime = System.currentTimeMillis();
        log.info("菜鸟电子面单查询传入参数：" + JSON.toJSONString(caiNiaoRequest));
        List<NameValuePair> list = new ArrayList<>();
        list.add(new BasicNameValuePair("logistics_interface", caiNiaoRequest.getLogistics_interface()));
        list.add(new BasicNameValuePair("logistic_provider_id", caiNiaoRequest.getLogistic_provider_id()));
        list.add(new BasicNameValuePair("msg_type", caiNiaoRequest.getMsg_type()));
        list.add(new BasicNameValuePair("data_digest", caiNiaoRequest.getData_digest()));
        log.info("【菜鸟发送短信传入参数】：{}", JSON.toJSONString(list));
        //http请求
        String res = HttpUtils.getPostResult(caiNiaoRequest.getUrl().trim(), list, "utf-8", "utf-8");

        log.info("【菜鸟电子面单查询返回参数：】{}", res);

        logService.install(ApiLog.builder()
                .requestMsg(JSON.toJSONString(caiNiaoRequest))
                .responseMsg(res)
                .url(caiNiaoRequest.getUrl())
                .exTime(new Long((System.currentTimeMillis() - startTime)).intValue())
                .createTime(new Date())
                .requestType(1)
                .platform(PlatformEnum.CaiNiao.getEnValue())
                .build());
        return res;
    }

    /**
     * 发送短信
     *
     * @param caiNiaoRequest
     * @return
     * @throws Exception
     */
    @RequestMapping("/smsSend")
    public Object smsSend(@RequestBody CaiNiaoRequest caiNiaoRequest) throws Exception {
        long startTime = System.currentTimeMillis();

        List<NameValuePair> list = new ArrayList<>();
        list.add(new BasicNameValuePair("logistics_interface", caiNiaoRequest.getLogistics_interface()));
        list.add(new BasicNameValuePair("logistic_provider_id", caiNiaoRequest.getLogistic_provider_id()));
        list.add(new BasicNameValuePair("msg_type", caiNiaoRequest.getMsg_type()));
        list.add(new BasicNameValuePair("data_digest", Md5Util.caiNiaoDoSign(caiNiaoRequest.getLogistics_interface(), "UTF-8", caiNiaoRequest.getData_digest())));
        log.info("【菜鸟发送短信传入参数】：{}", JSON.toJSONString(list));
        //http请求
        String result = HttpUtils.getPostResult(caiNiaoRequest.getUrl().trim(), list, "utf-8", "utf-8");
        log.info("【菜鸟发送短信返回参数：】{}", result);
        logService.install(ApiLog.builder()
                .requestMsg(JSON.toJSONString(caiNiaoRequest))
                .responseMsg(result)
                .url(caiNiaoRequest.getMsg_type())
                .exTime(new Long((System.currentTimeMillis() - startTime)).intValue())
                .createTime(new Date())
                .requestType(1)
                .platform("CaiNiao")
                .build());
        return result;
    }

    @PostMapping(value = "/callback/receive")
    public @ResponseBody
    String CallBackReceive(@RequestParam("logistics_interface") String logisticsInterface,
                           @RequestParam("data_digest") String dataGigest,
                           @RequestParam("msg_type") String msgType,
                           @RequestParam("msg_id") String msgId,
                           @RequestParam("partner_code") String partnerCode,
                           @RequestParam("from_code") String fromCode) {
        long startTime = System.currentTimeMillis();
        CaiNiaoReceiveRequest caiNiaoReceiveRequest = CaiNiaoReceiveRequest.builder()
                .logisticsInterface(logisticsInterface)
                .dataGigest(dataGigest)
                .msgType(msgType)
                .msgId(msgId)
                .partnerCode(partnerCode)
                .fromCode(fromCode)
                .build();
        String res = "";
        try {
            res = OkHttpUtil.postJson(sendmessageurl + "/sms/cainiao/receive", caiNiaoReceiveRequest);
            logService.install(ApiLog.builder()
                    .requestMsg(JSON.toJSONString(caiNiaoReceiveRequest))
                    .responseMsg(res)
                    .url("/logistics/cainiao/callback/receive")
                    .exTime(new Long((System.currentTimeMillis() - startTime)).intValue())
                    .createTime(new Date())
                    .requestType(2)
                    .platform("CaiNiao")
                    .build());
            log.info("【菜鸟短信回执】：{}", res);
        } catch (Exception e) {
            log.error("receive callback error :" + logisticsInterface, e);
            return "{\"success\":\"false\",\"errorCode\":\"100\",\"errorMsg\":\"系统错误\"}";
        }
        // 返回对象response ，JSON或者XML 取决于您服务注册的格式
        return res;
    }

    /**
     * 菜鸟猜面单接口
     *
     * @param request
     * @param response
     * @param caiNiaoRequest
     * @return
     * @throws Exception
     */
    @RequestMapping("/getExpressCompanyByExpCode")
    public Object getExpressCompanyByExpCode(HttpServletRequest request, HttpServletResponse response, @RequestBody CaiNiaoRequest caiNiaoRequest) throws Exception {
        long startTime = System.currentTimeMillis();
        log.info("菜鸟猜面单查询传入参数：" + JSON.toJSONString(caiNiaoRequest));
        List<NameValuePair> list = new ArrayList<>();
        list.add(new BasicNameValuePair("logistics_interface", caiNiaoRequest.getLogistics_interface()));
        list.add(new BasicNameValuePair("logistic_provider_id", caiNiaoRequest.getLogistic_provider_id()));
        list.add(new BasicNameValuePair("msg_type", caiNiaoRequest.getMsg_type()));
        list.add(new BasicNameValuePair("to_code", caiNiaoRequest.getTo_code()));
        list.add(new BasicNameValuePair("data_digest", caiNiaoRequest.getData_digest()));
        log.info("【菜鸟猜面单接口传入参数】：{}", JSON.toJSONString(list));
        //http请求
        String res = HttpUtils.getPostResult(caiNiaoRequest.getUrl().trim(), list, "utf-8", "utf-8");
        log.info("【菜鸟猜面单接口返回参数：】{}", res);
        logService.install(ApiLog.builder()
                .requestMsg(JSON.toJSONString(caiNiaoRequest))
                .responseMsg(res)
                .url(caiNiaoRequest.getUrl())
                .exTime(new Long((System.currentTimeMillis() - startTime)).intValue())
                .createTime(new Date())
                .requestType(1)
                .platform(PlatformEnum.CaiNiao.getEnValue())
                .build());
        return res;
    }

    /**
     * 快递入库,签收回传接口
     *
     * @param request
     * @param response
     * @param caiNiaoRequest
     * @return
     * @throws Exception
     */
    @RequestMapping("/caiNiaoSignBack")
    public Object caiNiaoSignBack(HttpServletRequest request, HttpServletResponse response, @RequestBody CaiNiaoRequest caiNiaoRequest) throws Exception {
        long startTime = System.currentTimeMillis();
        log.info("菜鸟回传快递信息接口传入参数：" + JSON.toJSONString(caiNiaoRequest));
        List<NameValuePair> list = new ArrayList<>();
        list.add(new BasicNameValuePair("logistics_interface", caiNiaoRequest.getLogistics_interface()));
        list.add(new BasicNameValuePair("logistic_provider_id", caiNiaoRequest.getLogistic_provider_id()));
        list.add(new BasicNameValuePair("msg_type", caiNiaoRequest.getMsg_type()));
        list.add(new BasicNameValuePair("to_code", caiNiaoRequest.getTo_code()));
        list.add(new BasicNameValuePair("data_digest", caiNiaoRequest.getData_digest()));
        log.info("【菜鸟回传快递信息接口传入参数】：{}", JSON.toJSONString(list));
        //http请求
        String res = HttpUtils.getPostResult(caiNiaoRequest.getUrl().trim(), list, "utf-8", "utf-8");
        log.info("【菜鸟回传快递信息接口返回参数：】{}", res);

        logService.install(ApiLog.builder()
                .requestMsg(JSON.toJSONString(caiNiaoRequest))
                .responseMsg(res)
                .url(caiNiaoRequest.getUrl())
                .exTime(new Long((System.currentTimeMillis() - startTime)).intValue())
                .createTime(new Date())
                .requestType(1)
                .platform(PlatformEnum.CaiNiao.getEnValue())
                .build());
        return res;
    }

    /**
     * 用户触达数据反馈接口(菜鸟签收回传回执)
     * @param logisticsInterface
     * @param dataGigest
     * @param msgType
     * @param msgId
     * @param partnerCode
     * @param fromCode
     * @return
     */
    @PostMapping(value = "/callback/receiveExpSignBack")
    public @ResponseBody
    String signBackReceive(@RequestParam("logistics_interface") String logisticsInterface,
                           @RequestParam("data_digest") String dataGigest,
                           @RequestParam("msg_type") String msgType,
                           @RequestParam("msg_id") String msgId,
                           @RequestParam("partner_code") String partnerCode,
                           @RequestParam("from_code") String fromCode) {
        long startTime = System.currentTimeMillis();
        CaiNiaoReceiveRequest caiNiaoReceiveRequest = CaiNiaoReceiveRequest.builder()
                .logisticsInterface(logisticsInterface)
                .dataGigest(dataGigest)
                .msgType(msgType)
                .msgId(msgId)
                .partnerCode(partnerCode)
                .fromCode(fromCode)
                .build();
        String res = "";
        log.info("参数:{}",caiNiaoReceiveRequest);
        try {
            res = OkHttpUtil.postJson(sendmessageurl + "/signBack/cainiao/receive", caiNiaoReceiveRequest);
            logService.install(ApiLog.builder()
                    .requestMsg(JSON.toJSONString(caiNiaoReceiveRequest))
                    .responseMsg(res)
                    .url("/logistics/cainiao/callback/receiveExpSignBack")
                    .exTime(new Long((System.currentTimeMillis() - startTime)).intValue())
                    .createTime(new Date())
                    .requestType(2)
                    .platform("CaiNiao")
                    .build());
            log.info("【菜鸟用户触达数据反馈回执】：{}", res);
        } catch (Exception e) {
            log.error("receive callback error :" + caiNiaoReceiveRequest, e);
            return "{\"success\":\"false\",\"errorCode\":\"100\",\"errorMsg\":\"系统错误\"}";
        }
        // 返回对象response ，JSON或者XML 取决于您服务注册的格式
        return res;
    }
}
