package com.bs.logisticslnfoapi.controller;

import com.alibaba.fastjson.JSON;
import com.bs.logisticslnfoapi.bean.ApiLog;
import com.bs.logisticslnfoapi.bean.jitu.JiTuRequestBean;
import com.bs.logisticslnfoapi.service.log.LogService;
import com.bs.logisticslnfoapi.util.OkHttpUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.Date;

/**
 * @author liujs
 * @desc 极兔接口
 */
@RestController
@Slf4j
@CrossOrigin("*")
@RequestMapping(value ="/logistics/jitu")
public class JiTuController {

    @Resource
    private LogService logService;

    /**
     * 根据运单号查询收件人手机号
     * @param jiTuRequestBean
     * @return
     * @throws Exception
     */
    @RequestMapping("/query/mobile")
    public String queryMobile(@RequestBody JiTuRequestBean jiTuRequestBean) throws Exception {
        long startTime = System.currentTimeMillis();
        String requestParams = JSON.toJSONString(jiTuRequestBean);
        log.info("极兔根据快递单号查询手机号接口传入参数:{}",requestParams);
        String res = OkHttpUtil.get(jiTuRequestBean.getUrl());
        log.info("极兔根据快递单号查询手机号接口返回结果:{}",res);
        logService.install(ApiLog.builder()
                .requestMsg(requestParams)
                .responseMsg(res)
                .url("/ecs/query/mobile")
                .exTime(new Long((System.currentTimeMillis() - startTime)).intValue())
                .createTime(new Date())
                .requestType(1)
                .platform("JT")
                .build());
         return res;
    }

    /**
     * 入库扫描
     * @param jiTuRequestBean
     * @return
     * @throws Exception
     */
    @RequestMapping("/enteringCabinetList/uploadData")
    public String inboundScanning(@RequestBody JiTuRequestBean jiTuRequestBean) throws Exception {
        long startTime = System.currentTimeMillis();
        String requestParams = JSON.toJSONString(jiTuRequestBean);
        log.info("极兔入库扫描接口传入参数:{}",requestParams);
        String res = OkHttpUtil.postJson(jiTuRequestBean.getUrl(),jiTuRequestBean.getJiTuDeliveryCabinetDTOList());
        log.info("极兔入库扫描接口返回结果:{}",res);
        logService.install(ApiLog.builder()
                .requestMsg(requestParams)
                .responseMsg(res)
                .url("/ecs/enteringCabinetList/uploadData")
                .exTime(new Long((System.currentTimeMillis() - startTime)).intValue())
                .createTime(new Date())
                .requestType(1)
                .platform("JT")
                .build());
        return res;
    }

    /**
     * 出库扫描
     * @param jiTuRequestBean
     * @return
     * @throws Exception
     */
    @RequestMapping("/cabinet/sign")
    public String outboundScanning(@RequestBody JiTuRequestBean jiTuRequestBean) throws Exception {
        long startTime = System.currentTimeMillis();
        String requestParams = JSON.toJSONString(jiTuRequestBean);
        log.info("极兔出库扫描接口传入参数:{}",requestParams);
        String res = OkHttpUtil.postJson(jiTuRequestBean.getUrl(),jiTuRequestBean.getJiTuDeliveryCabinetDTOList());
        log.info("极兔出库扫描接口返回结果:{}",res);
        logService.install(ApiLog.builder()
                .requestMsg(requestParams)
                .responseMsg(res)
                .url("/ecs/cabinet/sign")
                .exTime(new Long((System.currentTimeMillis() - startTime)).intValue())
                .createTime(new Date())
                .requestType(1)
                .platform("JT")
                .build());
        return res;
    }

    /**
     * 快件取出扫描(快递员/管理员取出重派件) --只是对接了这个接口 但是APP上没有实际应用
     * @param jiTuRequestBean
     * @return
     * @throws Exception
     */
    @RequestMapping("/cabinet/deliveryout")
    public String deliveryOutScanning(@RequestBody JiTuRequestBean jiTuRequestBean) throws Exception {
        long startTime = System.currentTimeMillis();
        String requestParams = JSON.toJSONString(jiTuRequestBean);
        log.info("极兔快件取出扫描接口传入参数:{}",requestParams);
        String res = OkHttpUtil.postJson(jiTuRequestBean.getUrl(),jiTuRequestBean.getJiTuDeliveryCabinetDTOList());
        log.info("极兔快件取出扫描接口返回结果:{}",res);
        logService.install(ApiLog.builder()
                .requestMsg(requestParams)
                .responseMsg(res)
                .url("/ecs/cabinet/deliveryout")
                .exTime(new Long((System.currentTimeMillis() - startTime)).intValue())
                .createTime(new Date())
                .requestType(1)
                .platform("JT")
                .build());
        return res;
    }
}
