package com.bs.logisticslnfoapi.controller;

import com.alibaba.fastjson.JSON;
import com.bs.logisticslnfoapi.bean.ApiLog;
import com.bs.logisticslnfoapi.bean.shentong.ShenTongRequestBean;
import com.bs.logisticslnfoapi.service.log.LogService;
import com.bs.logisticslnfoapi.util.Enum.PlatformEnum;
import com.sto.link.request.LinkRequest;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.Date;

import static com.sto.link.util.LinkUtils.request;

/**
 * @author liujs
 * @desc 申通接口
 */
@RestController
@Slf4j
@CrossOrigin("*")
@RequestMapping(value ="/logistics/shentong")
public class ShenTongController {

    @Resource
    private LogService logService;

    /**
     * 末端驿站列表
     * @param shentongRequestBean
     * @return
     */
    @RequestMapping("/site/getStationsList")
    public String getStationsList(@RequestBody ShenTongRequestBean shentongRequestBean){
        long startTime = System.currentTimeMillis();
        String requestParam = JSON.toJSONString(shentongRequestBean);
        LinkRequest data = getData(shentongRequestBean);
        log.info("申通末端驿站列表接口传入参数:{}",requestParam);
        String result = request(data,shentongRequestBean.getUrl(),shentongRequestBean.getSecretKey());
        log.info("申通末端驿站列表接口返回结果:{}",result);
        logService.install(ApiLog.builder()
                .requestMsg(requestParam)
                .responseMsg(result)
                .url("SITE_OPERATION_ENDPOINT_LIST")
                .exTime(new Long((System.currentTimeMillis() - startTime)).intValue())
                .createTime(new Date())
                .requestType(1)
                .platform(PlatformEnum.STO.getEnValue())
                .build());
        return result;
    }

    /**
     * 末端品牌信息查询
     * @param shentongRequestBean
     * @return
     */
    @RequestMapping("/site/getStationInfo")
    public String getStationInfo(@RequestBody ShenTongRequestBean shentongRequestBean){
        long startTime = System.currentTimeMillis();
        String requestParam = JSON.toJSONString(shentongRequestBean);
        LinkRequest data = getData(shentongRequestBean);
        log.info("申通末端品牌信息查询接口传入参数:{}",requestParam);
        String result = request(data,shentongRequestBean.getUrl(),shentongRequestBean.getSecretKey());
        log.info("申通末端品牌信息查询接口返回结果:{}",result);
        logService.install(ApiLog.builder()
                .requestMsg(requestParam)
                .responseMsg(result)
                .url("SITE_OPERATION_ENDPOINT_GET_INFO")
                .exTime(new Long((System.currentTimeMillis() - startTime)).intValue())
                .createTime(new Date())
                .requestType(1)
                .platform(PlatformEnum.STO.getEnValue())
                .build());
        return result;
    }

    /**
     * 末端品牌驿站注册
     * @param shentongRequestBean
     * @return
     */
    @RequestMapping("/site/registerStore")
    public String registerStore(@RequestBody ShenTongRequestBean shentongRequestBean){
        long startTime = System.currentTimeMillis();
        String requestParam = JSON.toJSONString(shentongRequestBean);
        LinkRequest data = getData(shentongRequestBean);
        log.info("申通末端品牌驿站注册接口传入参数:{}",requestParam);
        String result = request(data,shentongRequestBean.getUrl(),shentongRequestBean.getSecretKey());
        log.info("申通末端品牌驿站注册接口返回结果:{}",result);
        logService.install(ApiLog.builder()
                .requestMsg(requestParam)
                .responseMsg(result)
                .url("SITE_OPERATION_ENDPOINT_REGISTER")
                .exTime(new Long((System.currentTimeMillis() - startTime)).intValue())
                .createTime(new Date())
                .requestType(1)
                .platform(PlatformEnum.STO.getEnValue())
                .build());
        return result;
    }

    /**
     * 末端品牌驿站信息修改
     * @param shentongRequestBean
     * @return
     */
    @RequestMapping("/site/updateStoreInfo")
    public String updateStoreInfo(@RequestBody ShenTongRequestBean shentongRequestBean){
        long startTime = System.currentTimeMillis();
        String requestParam = JSON.toJSONString(shentongRequestBean);
        LinkRequest data = getData(shentongRequestBean);
        log.info("申通末端品牌驿站信息修改接口传入参数:{}",requestParam);
        String result = request(data,shentongRequestBean.getUrl(),shentongRequestBean.getSecretKey());
        log.info("申通末端品牌驿站信息修改接口返回结果:{}",result);
        logService.install(ApiLog.builder()
                .requestMsg(requestParam)
                .responseMsg(result)
                .url("SITE_OPERATION_ENDPOINT_MODIFY")
                .exTime(new Long((System.currentTimeMillis() - startTime)).intValue())
                .createTime(new Date())
                .requestType(1)
                .platform(PlatformEnum.STO.getEnValue())
                .build());
        return result;
    }

    /**
     * 末端品牌状态修改
     * @param shentongRequestBean
     * @return
     */
    @RequestMapping("/site/changeStatus")
    public String changeStatus(@RequestBody ShenTongRequestBean shentongRequestBean){
        long startTime = System.currentTimeMillis();
        String requestParam = JSON.toJSONString(shentongRequestBean);
        LinkRequest data = getData(shentongRequestBean);
        log.info("申通末端品牌状态修改接口传入参数:{}",requestParam);
        String result = request(data,shentongRequestBean.getUrl(),shentongRequestBean.getSecretKey());
        log.info("申通末端品牌状态修改接口返回结果:{}",result);
        logService.install(ApiLog.builder()
                .requestMsg(requestParam)
                .responseMsg(result)
                .url("SITE_OPERATION_ENDPOINT_CHANGE_STATUS")
                .exTime(new Long((System.currentTimeMillis() - startTime)).intValue())
                .createTime(new Date())
                .requestType(1)
                .platform(PlatformEnum.STO.getEnValue())
                .build());
        return result;
    }

    /**
     * 驿站类扫描数据上传接口
     * @param shentongRequestBean
     * @return
     */
    @RequestMapping("/scanAndUplodaMsgByStation")
    public String scanAndUplodaMsgByStation(@RequestBody ShenTongRequestBean shentongRequestBean){
        long startTime = System.currentTimeMillis();
        String requestParam = JSON.toJSONString(shentongRequestBean);
        LinkRequest data = getData(shentongRequestBean);
        log.info("申通驿站类扫描数据上传接口接口传入参数:{}",requestParam);
        String result = request(data,shentongRequestBean.getUrl(),shentongRequestBean.getSecretKey());
        log.info("申通驿站类扫描数据上传接口接口返回结果:{}",result);
        logService.install(ApiLog.builder()
                .requestMsg(requestParam)
                .responseMsg(result)
                .url("GALAXY_RECEIVE_SCAN_MESSAGE_STATION")
                .exTime(new Long((System.currentTimeMillis() - startTime)).intValue())
                .createTime(new Date())
                .requestType(1)
                .platform(PlatformEnum.STO.getEnValue())
                .build());
        return result;
    }

    private LinkRequest getData(ShenTongRequestBean shentongRequestBean) {
        LinkRequest linkRequest = new LinkRequest();
        linkRequest.setFromAppkey(shentongRequestBean.getFrom_appkey());
        linkRequest.setFromCode(shentongRequestBean.getFrom_code());
        linkRequest.setToAppkey(shentongRequestBean.getTo_appkey());
        linkRequest.setToCode(shentongRequestBean.getTo_code());
        linkRequest.setApiName(shentongRequestBean.getApi_name());
        linkRequest.setContent(shentongRequestBean.getContent());
        return linkRequest;
    }
}
