package com.bs.logisticslnfoapi.factory.smsPlatform.realize;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.aliyuncs.CommonRequest;
import com.aliyuncs.CommonResponse;
import com.aliyuncs.DefaultAcsClient;
import com.aliyuncs.IAcsClient;
import com.aliyuncs.http.MethodType;
import com.aliyuncs.profile.DefaultProfile;
import com.bs.logisticslnfoapi.bean.ApiLog;
import com.bs.logisticslnfoapi.bean.alibb.AliBaBaSmsStatusReportBean;
import com.bs.logisticslnfoapi.bean.alibb.AlibbQuerySmsDetailsRequest;
import com.bs.logisticslnfoapi.bean.alibb.AlibbSendSmsRequest;
import com.bs.logisticslnfoapi.factory.smsPlatform.BaseSmsPlatform;
import com.bs.logisticslnfoapi.service.log.LogService;
import com.bs.logisticslnfoapi.util.OkHttpUtil;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;

/**
 * @PackageName: com.bs.logisticslnfoapi.factory.smsPlatform.realize
 * @ClassName: ALiBaBa
 * @Description: TODO
 * @Author: liujs
 * @Date: 2020/05/20 下午 16:45
 * @Version: 1.0
 */
@Slf4j
@Component("ALBB")
@RestController
public class AliBaBa implements BaseSmsPlatform {

    @Resource
    private LogService logService;

    @Value("${sendmessageurl}")
    private String sendMessageUrl;

    @Override
    public String sendSms(String params) throws Exception {
        long startTime = System.currentTimeMillis();
        CommonRequest commonRequest = new CommonRequest();
        AlibbSendSmsRequest request = JSONObject.parseObject(params,AlibbSendSmsRequest.class);
        IAcsClient client = getIAcsClient(request.getAccessKeyId(), request.getAppSecret());
        //系统参数
        commonRequest.setDomain(request.getUrl());
        commonRequest.setVersion("2017-05-25");
        commonRequest.setAction("SendSms");
        //业务参数
        commonRequest.putQueryParameter("RegionId", "cn-hangzhou");
        commonRequest.putQueryParameter("PhoneNumbers",request.getPhoneNumbers());
        commonRequest.putQueryParameter("SignName",request.getSignName());
        commonRequest.putQueryParameter("TemplateCode",request.getTemplateCode());
        commonRequest.putQueryParameter("TemplateParam",JSON.toJSONString(request.getTemplateParam()));
        CommonResponse response = client.getCommonResponse(commonRequest);
        log.info("【阿里云发送短信接口返回参数】：{}",response.getData());
        logService.install(ApiLog.builder()
                .requestMsg(params)
                .responseMsg(response.getData())
                .url(request.getUrl())
                .exTime(new Long((System.currentTimeMillis() - startTime)).intValue())
                .createTime(new Date())
                .requestType(1)
                .platform("ALBB")
                .build());
        return response.getData();
    }

    /**
     * 阿里云短信回执接口
     * @param params
     * @return
     * @throws Exception
     */
    @RequestMapping("/smsStatusReportByAliBaBa")
    public String smsStatusReport(@RequestBody List<AliBaBaSmsStatusReportBean> params) {
        long startTime = System.currentTimeMillis();
        //TODO
        String url = sendMessageUrl + "/smsStatusReportByALBB";
        log.info("阿里云短信回执传入参数为:{}",JSON.toJSONString(params));
        String result;
        try {
            result = OkHttpUtil.postJson(url, params);
            log.info("【阿里云短信回执返回参数】：{}", result);
            logService.install(ApiLog.builder()
                    .requestMsg(JSON.toJSONString(params))
                    .responseMsg(result)
                    .url(url)
                    .exTime(new Long((System.currentTimeMillis() - startTime)).intValue())
                    .createTime(new Date())
                    .requestType(2)
                    .platform("ALBB")
                    .build());
        }catch (Exception e){
            return "{\"code\":-1,\"msg\":\"系统异常\"}";
        }
        return result;
    }

    /**
     * 阿里巴巴短信结果查询
     * @param request
     * @return
     * @throws Exception
     */
    @RequestMapping("/querySendDetails")
    public Object alibbSmsQueryDetails(@RequestBody AlibbQuerySmsDetailsRequest request) throws Exception{
        long startTime = System.currentTimeMillis();
        CommonRequest commonRequest = new CommonRequest();
        IAcsClient client = getIAcsClient(request.getAccessKeyId(), request.getAppSecret());
        //系统参数
        commonRequest.setMethod(MethodType.POST);
        commonRequest.setDomain(request.getUrl());
        commonRequest.setVersion("2017-05-25");
        commonRequest.setAction("QuerySendDetails");
        commonRequest.putQueryParameter("RegionId", "cn-hangzhou");
        commonRequest.putQueryParameter("CurrentPage",request.getCurrentPage().toString());
        commonRequest.putQueryParameter("PageSize",request.getPageSize().toString());
        commonRequest.putQueryParameter("PhoneNumber",request.getPhoneNumber());
        commonRequest.putQueryParameter("SendDate",request.getSendDate());
        commonRequest.putQueryParameter("AccessKeyId",request.getAccessKeyId());
        commonRequest.putQueryParameter("Action","QuerySendDetails");
        commonRequest.putQueryParameter("BizId",request.getBizId());
        CommonResponse response = client.getCommonResponse(commonRequest);
        log.info("【ALiBaBa平台短信回执接口返回参数】：{}",response.getData());
        logService.install(ApiLog.builder()
                .requestMsg(JSON.toJSONString(request))
                .responseMsg(response.getData())
                .url(request.getUrl())
                .exTime(new Long((System.currentTimeMillis() - startTime)).intValue())
                .createTime(new Date())
                .requestType(1)
                .platform("ALBB")
                .build());
        return response.getData();
    }

    private IAcsClient getIAcsClient(String accessKeyId, String appSecret) {
        DefaultProfile profile = DefaultProfile.getProfile("cn-hangzhou", accessKeyId, appSecret);
        return new DefaultAcsClient(profile);
    }
}
