package com.bs.logisticslnfoapi.service.jhy.impl;

import com.alibaba.fastjson.JSON;
import com.bs.logisticslnfoapi.bean.ApiLog;
import com.bs.logisticslnfoapi.bean.ResultBean;
import com.bs.logisticslnfoapi.bean.jhy.JhyPhone;
import com.bs.logisticslnfoapi.bean.jhy.JhyPhonePushRequest;
import com.bs.logisticslnfoapi.bean.jhy.JhyPhoneStateRequest;
import com.bs.logisticslnfoapi.exceptions.APIBusinessException;
import com.bs.logisticslnfoapi.service.jhy.JhyService;
import com.bs.logisticslnfoapi.service.log.LogService;
import com.bs.logisticslnfoapi.util.AESUtil;
import com.bs.logisticslnfoapi.util.Enum.PlatformEnum;
import com.bs.logisticslnfoapi.util.HttpUtils;
import com.bs.logisticslnfoapi.util.OkHttpUtil;
import com.google.gson.Gson;
import lombok.extern.slf4j.Slf4j;
import org.apache.http.NameValuePair;
import org.apache.http.message.BasicNameValuePair;
import org.apache.logging.log4j.util.Strings;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import sun.misc.BASE64Decoder;
import sun.misc.BASE64Encoder;

import java.util.*;

/**
 * @ClassName JhyServiceImpl
 * @Description 居欢颜
 * @Author Administrator
 * @Date 2019/8/5 0005 下午 14:31
 * @Version 1.0
 **/
@Service
@Slf4j
public class JhyServiceImpl implements JhyService {

    @Value("${hdurl}")
    private String url;

    @Value("${jhyscrect}")
    private String jhyscrect;

    @Value("${jhurl}")
    private String jhurl;

    @Autowired
    private LogService logService;


    @Override
    public ResultBean<Boolean> addPhone(String data) throws Exception {
        long startTime = System.currentTimeMillis();
        BASE64Decoder decoder = new BASE64Decoder();
        data = new String(decoder.decodeBuffer(data));
        if(Strings.isBlank(data)){
            throw new APIBusinessException("data参数错误");
        }
        JhyPhone jhyPhone = new Gson().fromJson(data,JhyPhone.class);
        if(jhyPhone == null){
            throw new APIBusinessException("传入参数不能为空");
        }
        if(Strings.isBlank(jhyPhone.getPhone())){
            throw new APIBusinessException("传入电话号码不能为空");
        }
        List<NameValuePair> list = new ArrayList<>();
        list.add(new BasicNameValuePair("json", data));
        //http请求
        String result = HttpUtils.getPostResult(url.trim(), list,"utf-8","utf-8");
        logService.install(ApiLog.builder()
                .requestMsg(data)
                .responseMsg(result)
                .url("/logistics/jhy/phone/add")
                .exTime(new Long((System.currentTimeMillis() - startTime)).intValue())
                .createTime(new Date())
                .requestType(2)
                .platform(PlatformEnum.JHY.getEnValue())
                .build());
        if(Strings.isBlank(result)){
            throw new Exception("系统错误");
        }
        return new Gson().fromJson(result,ResultBean.class);
    }

    @Override
    public String pushPhone(JhyPhonePushRequest jhyPhonePushRequest) throws Exception {
        String data = new String(AESUtil.encrypt(new Gson().toJson(jhyPhonePushRequest), jhyscrect));
        return jhyPostResult(data,"/express/info",new Gson().toJson(jhyPhonePushRequest));
    }

    @Override
    public String pushSign(JhyPhoneStateRequest jhyPhoneStateRequest) throws Exception {
        String data = new String(AESUtil.encrypt(new Gson().toJson(jhyPhoneStateRequest), jhyscrect));
        return jhyPostResult(data,"/express/state",new Gson().toJson(jhyPhoneStateRequest));
    }

    private String jhyPostResult(String data,String burl,String request) throws Exception {
        long startTime = System.currentTimeMillis();
        Map map = new HashMap();
        map.put("data",data);
        String result = OkHttpUtil.postJson(jhurl.trim()+burl, map);
        log.info("调用居欢颜接口："+jhurl.trim()+burl+"，返回参数："+result);
        logService.install(ApiLog.builder()
                .requestMsg(request)
                .responseMsg(result)
                .url(burl)
                .exTime(new Long((System.currentTimeMillis() - startTime)).intValue())
                .createTime(new Date())
                .requestType(1)
                .platform(PlatformEnum.JHY.getEnValue())
                .build());
        return result;
    }
}
