package com.bs.logisticslnfoapi.util;

import org.apache.http.HttpEntity;
import org.apache.http.NameValuePair;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.client.utils.URIBuilder;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.util.EntityUtils;

import java.io.*;
import java.net.URL;
import java.net.URLConnection;
import java.net.URLEncoder;
import java.util.List;
import java.util.Map;

/**
 * get\post请求工具类
 *
 * @author XieXiCai
 * @created 2014-6-4下午10:59:03
 */
public class HttpUtils {

    /**
     * 发送 get请求
     * @param host
     * @param param
     * @param encoding 参数编码
     * @param decoding 返回结果编码
     * @return
     */
    public static String sendGetReq(String host, String param, String encoding, String decoding) throws Exception{
        String result = "";
        BufferedReader reader = null;
        InputStream inputStream = null;

        try {
            param = param == null ? "" :param;
            param = isEmpty(encoding) ? param : URLEncoder.encode(param, encoding);

            URL url = new URL(host + param);
            URLConnection conn = url.openConnection();
            conn.setRequestProperty("source","YTO-STAGE");
            inputStream = conn.getInputStream();
            reader = new BufferedReader(new InputStreamReader(inputStream,  decoding));

            String line = null;
            while((line = reader.readLine()) != null) {
                result += line;
            }
        } catch (Exception e) {
            throw e;
        } finally{
            if(reader != null) reader.close();
            if(reader != null) inputStream.close();
        }
        return result;
    }

    /**
     * 发送post请求
     * @param host
     * @param param
     * @param encoding
     * @param decoding
     * @return
     */
    public static String sendPostReq(String host, Map<String, String> param, String encoding, String decoding) throws Exception{
        String result = "";
        BufferedReader reader = null;
        InputStream inputStream = null;
        StringBuffer buffer = new StringBuffer();
        try {
            String request = "";
            if(param != null && param.size()>0){
                for(String str : param.keySet()){
                    buffer.append(str).append("=").append(param.get(str)).append("&");
                }

                //去掉最后一个&并urlencode
                request = buffer.toString().substring(0,buffer.toString().length() - 1);
            }

            URL url = new URL(host);
            // 打开和URL之间的连接
            URLConnection conn = url.openConnection();
            // 发送POST请求必须设置
            conn.setDoOutput(true);
            // 获取URLConnection对象对应的输出流
            conn.getOutputStream().write(isEmpty(encoding) ? request.getBytes("utf-8") : request.getBytes(encoding));
            // flush输出流的缓冲
            conn.getOutputStream().flush();
            conn.getOutputStream().close();

            // 解析返回结果
            inputStream = conn.getInputStream();
            reader = new BufferedReader(new InputStreamReader(inputStream,  decoding));
            String line = null;
            while((line = reader.readLine()) != null) {
                result += line;
            }
        } catch (Exception e) {
            throw e;
        } finally{
            if(reader != null) reader.close();
            if(reader != null) inputStream.close();
        }
        return result;
    }

    /**
     * httppPost请求
     *
     * @param url
     * @param list
     * @return
     */
    public static String getPostResult(String url, List<NameValuePair> list,String encoding, String decoding) throws Exception{
        CloseableHttpClient httpClient = HttpClients.createDefault();
        try {
            HttpPost post = new HttpPost(url);
            post.setEntity(new UrlEncodedFormEntity(list, encoding));
            CloseableHttpResponse response = httpClient.execute(post);
            HttpEntity entity = response.getEntity();
            String content = EntityUtils.toString(entity, decoding);
            response.close();
            return content;
        } catch (IOException e) {
            throw e;
        } finally {
            try {
                httpClient.close();
            } catch (IOException e) {
                throw e;
            }
        }
    }

    /**
     * @param url
     * @param param
     * @param connectionTimeout 连接超时时间,为0不限
     * @param readTimeout       读取超时时间,为0不限
     */
    public static String sendPost(String url, String param, int connectionTimeout, int readTimeout, Map<String, String> headers) {
        PrintWriter out = null;
        BufferedReader in = null;
        String result = "";
        try {
            URL realUrl = new URL(url);
            // 打开和URL之间的连接
            URLConnection conn = realUrl.openConnection();
            if(connectionTimeout > 0){
                conn.setConnectTimeout(connectionTimeout);
            }
            if(readTimeout > 0){
                conn.setReadTimeout(readTimeout);
            }
            // 设置通用的请求属性
            conn.setRequestProperty("accept", "*/*");
            conn.setRequestProperty("connection", "Keep-Alive");
            conn.setRequestProperty("user-agent",
                    "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1;SV1)");
            if(headers != null && headers.size() > 0){
                for (Map.Entry<String, String> map : headers.entrySet()) {
                    conn.setRequestProperty(map.getKey(), map.getValue());
                }
            }

            // 发送POST请求必须设置如下两行
            conn.setDoOutput(true);
            conn.setDoInput(true);
            // 获取URLConnection对象对应的输出流
            out = new PrintWriter(conn.getOutputStream());
            // 发送请求参数
            out.print(param);
            // flush输出流的缓冲
            out.flush();
            // 定义BufferedReader输入流来读取URL的响应
            in = new BufferedReader(
                    new InputStreamReader(conn.getInputStream()));
            String line;
            while ((line = in.readLine()) != null) {
                result += line;
            }
        } catch (Exception e) {
            System.out.println("发送 POST 请求出现异常！"+e);
            e.printStackTrace();
        }
        //使用finally块来关闭输出流、输入流
        finally{
            try{
                if(out!=null){
                    out.close();
                }
                if(in!=null){
                    in.close();
                }
            }
            catch(IOException ex){
                ex.printStackTrace();
            }
        }
        return result;
    }

    /**
     * 判断字符串是否为空
     * @param str
     * @return
     * @author XieXiCai
     * @created 2016年10月12日下午8:52:55
     */
    private static boolean isEmpty(String str){
        if(str == null || "".equals(str))
            return true;
        return false;
    }

    /**
     * 发送post请求
     * @param host
     * @param param
     * @param encoding
     * @param decoding
     * Content-Type application/json
     * @return
     */
    public static String sendPostRequest(String host, String param, String encoding, String decoding) throws Exception{
        String result = "";
        BufferedReader reader = null;
        InputStream inputStream = null;

        try {
            URL url = new URL(host);
            // 打开和URL之间的连接
            URLConnection conn = url.openConnection();
            conn.setRequestProperty("Content-Type", "application/json");
            // 发送POST请求必须设置
            conn.setDoOutput(true);
            // 获取URLConnection对象对应的输出流
            conn.getOutputStream().write(isEmpty(encoding) ? param.getBytes("utf-8") : param.getBytes(encoding));
            // flush输出流的缓冲
            conn.getOutputStream().flush();
            conn.getOutputStream().close();

            // 解析返回结果
            inputStream = conn.getInputStream();
            reader = new BufferedReader(new InputStreamReader(inputStream,  decoding));
            String line = null;
            while((line = reader.readLine()) != null) {
                result += line;
            }
        } catch (Exception e) {
            throw e;
        } finally{
            if(reader != null) reader.close();
            if(reader != null) inputStream.close();
        }
        return result;
    }
}

