package com.bs.logisticslnfoapi.util.rabbit;

import lombok.Data;
import org.springframework.amqp.core.Binding;
import org.springframework.amqp.core.BindingBuilder;
import org.springframework.amqp.core.DirectExchange;
import org.springframework.amqp.core.Queue;
import org.springframework.amqp.rabbit.connection.CachingConnectionFactory;
import org.springframework.amqp.rabbit.connection.ConnectionFactory;
import org.springframework.amqp.rabbit.core.RabbitAdmin;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.amqp.RabbitProperties;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.stereotype.Component;

/**
 * @ClassName RabbitConfig
 * @Description rabbit参数
 * @Author kael
 * @Date 2019/9/27 0027 上午 11:28
 * @Version 1.0
 **/
@Data
@Component
@ConfigurationProperties(prefix = "spring.rabbitmq")
@Configuration
public class RabbitConfig {

    //实例Id 从阿里云AMQP控制台获取
    private static final String  INSTANCE_ID="1128688346197095";

    @Autowired
    private RabbitProperties rabbitProperties;

    private String apiLogQueueName;
    private String apiLogExchangeName;
    private String apiLogRoutingKey;

    @Bean
    public ConnectionFactory getConnectionFactory() {
        com.rabbitmq.client.ConnectionFactory rabbitConnectionFactory = new com.rabbitmq.client.ConnectionFactory();
        rabbitConnectionFactory.setHost(rabbitProperties.getHost());
        rabbitConnectionFactory.setPort(rabbitProperties.getPort());
        rabbitConnectionFactory.setVirtualHost(rabbitProperties.getVirtualHost());

        AliyunCredentialsProvider credentialsProvider = new AliyunCredentialsProvider(rabbitProperties.getUsername(), rabbitProperties.getPassword(), INSTANCE_ID);
        rabbitConnectionFactory.setCredentialsProvider(credentialsProvider);
        rabbitConnectionFactory.setAutomaticRecoveryEnabled(true);
        rabbitConnectionFactory.setNetworkRecoveryInterval(5000);
        ConnectionFactory connectionFactory = new CachingConnectionFactory(rabbitConnectionFactory);
        ((CachingConnectionFactory)connectionFactory).setPublisherConfirms(rabbitProperties.isPublisherConfirms());
        ((CachingConnectionFactory)connectionFactory).setPublisherReturns(rabbitProperties.isPublisherReturns());
        return connectionFactory;
    }


    @Bean
    public RabbitAdmin rabbitAdmin(ConnectionFactory getConnectionFactory) {
        RabbitAdmin rabbitAdmin = new RabbitAdmin(getConnectionFactory);
        //设置忽略声明异常
        rabbitAdmin.setIgnoreDeclarationExceptions(true);
        return rabbitAdmin;
    }

    // 1.定义短信队列
    @Bean
    public Queue fanSmsThirtyQueue() {
        return new Queue(apiLogQueueName,true);
    }

    // 2.定义交换机
    @Bean
    DirectExchange smsThirtyExchange() {
        return new DirectExchange(apiLogExchangeName,true,false);
    }


    // 3.队列与交换机绑定短信队列
    @Bean
    public Binding bindingSMSThirtyExchange(Queue fanSmsThirtyQueue, DirectExchange smsThirtyExchange) {
        return BindingBuilder.bind(fanSmsThirtyQueue).to(smsThirtyExchange).with(apiLogRoutingKey);
    }

}
